unit History;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Clipbrd;

type
  TFormHistory = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

  private
    FActive: Boolean;
    FFirstTime: Boolean;
    FList: TStringList;
    FLimit: Integer;
    FDelay: Integer;
    FMaxSize: Integer;
    FNextWnd: HWND;

    function GetCount: Integer;
    function GetList(Index: Integer): string;

    procedure SetLimit(const Value: Integer);
    procedure SetMaxSize(const Value: Integer);

    procedure WMChangeCBChain(var Msg: TWMChangeCBChain); message WM_CHANGECBCHAIN;
    procedure WMDrawClipboard(var Msg: TWMDrawClipboard); message WM_DRAWCLIPBOARD;

  protected
    procedure CreateParams(var Params: TCreateParams); override;

  public
    procedure Start;
    procedure Stop;
    procedure Delete(Index: Integer);
    procedure Add(Text: String);

    property IsActive: Boolean read FActive;
    property List[Index: Integer]: string read GetList;
    property Count: Integer read GetCount;
    property Limit: Integer read FLimit write SetLimit;
    property Delay: Integer read FDelay write FDelay;
    property MaxSize: Integer read FMaxSize write SetMaxSize;

  end;

var
  FormHistory: TFormHistory;

implementation

{$R *.dfm}

{ TFormHistory }

procedure TFormHistory.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.WndParent := HWND_MESSAGE;
end;

procedure TFormHistory.FormCreate(Sender: TObject);
begin
  //m
  FList := TStringList.Create;
end;

procedure TFormHistory.FormDestroy(Sender: TObject);
begin
  //~
  Stop;
  //j
  FList.Free;
end;

procedure TFormHistory.Delete(Index: Integer);
begin
  //폜
  FList.Delete(Index);
end;

procedure TFormHistory.Add(Text: String);
begin
  //ǉ
  FList.Add(Text);
end;

function TFormHistory.GetCount: Integer;
begin
  //s
  Result := FList.Count;
end;

function TFormHistory.GetList(Index: Integer): string;
begin
  //擾
  Result := FList[Index];
end;

procedure TFormHistory.SetLimit(const Value: Integer);
var
  iRow1: Integer;
begin
  //ݒ
  FLimit := Value;
  //Ōォ폜
  for iRow1 := FList.Count-1 downto FLimit do
  begin
    //폜
    FList.Delete(iRow1);
  end;
end;

procedure TFormHistory.SetMaxSize(const Value: Integer);
begin
  //͈͊OȂ
  if (Value = 0) or (Value >= MaxInt) then
  begin
    //ől
    FMaxSize := MaxInt;
  end
  else
  begin
    //wl
    FMaxSize := Value;
  end;
end;

procedure TFormHistory.Start;
begin
  //N
  if FActive = FALSE then
  begin
    //ݒ
    FFirstTime := TRUE;
    //Nbv{[hr[ɓo^āA擾
    FNextWnd := SetClipboardViewer(Handle);
    //ݒ
    FActive := TRUE;
  end;
end;

procedure TFormHistory.Stop;
begin
  //Nς
  if FActive = TRUE then
  begin
    //Nbv{[hr[폜āÃnho^
    ChangeClipboardChain(Handle, FNextWnd);
    //ݒ
    FActive := FALSE;
  end;
end;

//Nbv{[h̃`FCꂽ
procedure TFormHistory.WMChangeCBChain(var Msg: TWMChangeCBChain);
begin
  //̃nh폜ꂽ
  if      Msg.Remove = FNextWnd then
  begin
    //̎̃nh擾
    FNextWnd := Msg.Next
  end
  //̃nh
  else if FNextWnd <> 0 then
  begin
    //bZ[W𑗂
    SendMessage(FNextWnd, WM_CHANGECBCHAIN, Msg.Remove, Msg.Next);
  end;
end;

//Nbv{[hꂽ
procedure TFormHistory.WMDrawClipboard(var Msg: TWMDrawClipboard);
const
  CF_OTHERS = 0;
var
  Text: string;
  CbFormat: Integer;
  CbHandle: HGLOBAL;
begin
  //ŏɎo^
  if FFirstTime = TRUE then
  begin
    //Zbg
    FFirstTime := FALSE;
    //f
    Exit;
  end
  //ŏ̓o^ȊO
  else
  begin
    //ҋ@
    Sleep(FDelay);
    //I[v
    ClipBoard.Open;

    //^𔻒
    if Clipboard.HasFormat(CF_UNICODETEXT) then
    begin
      CbFormat := CF_UNICODETEXT;
    end
    else if Clipboard.HasFormat(CF_TEXT) then
    begin
      CbFormat := CF_TEXT;
    end
    else
    begin
      CbFormat := CF_OTHERS;
    end;

    //҂ĂtH[}bg
    if CbFormat <> CF_OTHERS then
    begin
      //nh擾
      CbHandle := Clipboard.GetAsHandle(CbFormat);
      try
        //TCYݒȏ
        if GlobalSize(CbHandle) > Cardinal(FMaxSize) then
        begin
          //ݒ肵Ȃ
          Text := '';
        end
        else
        begin
          //擾
          Text := Clipboard.AsText;
        end;
      finally
        //j
        GlobalUnlock(CbHandle);
      end;
    end;
    //
    Clipboard.Close;

    //񂪂       Xg         Xg̐擪ƓeقȂ
    if (Text <> '') and ((Flist.Count = 0) or (FList[0] <> Text)) then
    begin
      //擪ɒǉ
      FList.Insert(0, Text);
      //𐔂𒴂
      if FList.Count > FLimit then
      begin
        //Ō폜
        FList.Delete(FLimit);
      end;
    end;
  end;

  //o^Ă
  if FNextWnd <> 0 then
  begin
    //̃EBhEɓbZ[W𑗂
    SendMessage(FNextWnd, WM_DRAWCLIPBOARD, 0, 0);
  end;
end;

end.
